# BambangShop Publisher App
Tutorial and Example for Advanced Programming 2024 - Faculty of Computer Science, Universitas Indonesia

---

## About this Project
In this repository, we have provided you a REST (REpresentational State Transfer) API project using Rocket web framework.

This project consists of four modules:
1.  `controller`: this module contains handler functions used to receive request and send responses.
    In Model-View-Controller (MVC) pattern, this is the Controller part.
2.  `model`: this module contains structs that serve as data containers.
    In MVC pattern, this is the Model part.
3.  `service`: this module contains structs with business logic methods.
    In MVC pattern, this is also the Model part.
4.  `repository`: this module contains structs that serve as databases and methods to access the databases.
    You can use methods of the struct to get list of objects, or operating an object (create, read, update, delete).

This repository provides a basic functionality that makes BambangShop work: ability to create, read, and delete `Product`s.
This repository already contains a functioning `Product` model, repository, service, and controllers that you can try right away.

As this is an Observer Design Pattern tutorial repository, you need to implement another feature: `Notification`.
This feature will notify creation, promotion, and deletion of a product, to external subscribers that are interested of a certain product type.
The subscribers are another Rocket instances, so the notification will be sent using HTTP POST request to each subscriber's `receive notification` address.

## API Documentations

You can download the Postman Collection JSON here: https://ristek.link/AdvProgWeek7Postman

After you download the Postman Collection, you can try the endpoints inside "BambangShop Publisher" folder.
This Postman collection also contains endpoints that you need to implement later on (the `Notification` feature).

Postman is an installable client that you can use to test web endpoints using HTTP request.
You can also make automated functional testing scripts for REST API projects using this client.
You can install Postman via this website: https://www.postman.com/downloads/

## How to Run in Development Environment
1.  Set up environment variables first by creating `.env` file.
    Here is the example of `.env` file:
    ```bash
    APP_INSTANCE_ROOT_URL="http://localhost:8000"
    ```
    Here are the details of each environment variable:
    | variable              | type   | description                                                |
    |-----------------------|--------|------------------------------------------------------------|
    | APP_INSTANCE_ROOT_URL | string | URL address where this publisher instance can be accessed. |
2.  Use `cargo run` to run this app.
    (You might want to use `cargo check` if you only need to verify your work without running the app.)

## Mandatory Checklists (Publisher)
-   [v] Clone https://gitlab.com/ichlaffterlalu/bambangshop to a new repository.
-   **STAGE 1: Implement models and repositories**
    -   [v] Commit: `Create Subscriber model struct.`
    -   [v] Commit: `Create Notification model struct.`
    -   [v] Commit: `Create Subscriber database and Subscriber repository struct skeleton.`
    -   [v] Commit: `Implement add function in Subscriber repository.`
    -   [v] Commit: `Implement list_all function in Subscriber repository.`
    -   [v] Commit: `Implement delete function in Subscriber repository.`
    -   [v] Write answers of your learning module's "Reflection Publisher-1" questions in this README.
-   **STAGE 2: Implement services and controllers**
    -   [v] Commit: `Create Notification service struct skeleton.`
    -   [v] Commit: `Implement subscribe function in Notification service.`
    -   [v] Commit: `Implement subscribe function in Notification controller.`
    -   [v] Commit: `Implement unsubscribe function in Notification service.`
    -   [v] Commit: `Implement unsubscribe function in Notification controller.`
    -   [v] Write answers of your learning module's "Reflection Publisher-2" questions in this README.
-   **STAGE 3: Implement notification mechanism**
    -   [v] Commit: `Implement update method in Subscriber model to send notification HTTP requests.`
    -   [v] Commit: `Implement notify function in Notification service to notify each Subscriber.`
    -   [v] Commit: `Implement publish function in Program service and Program controller.`
    -   [v] Commit: `Edit Product service methods to call notify after create/delete.`
    -   [v] Write answers of your learning module's "Reflection Publisher-3" questions in this README.

## Your Reflections
This is the place for you to write reflections:

### Mandatory (Publisher) Reflections

#### Reflection Publisher-1

1. In the Observer pattern diagram explained by the Head First Design Pattern book, Subscriber is defined as an interface. Explain based on your understanding of Observer design patterns, do we still need an interface (or trait in Rust) in this BambangShop case, or a single Model struct is enough?

A trait is used when dealing with different types of subscriber that would require a different implementation, but since all subscriber in bambangshop seems to work in the same way, i.e. receiving notification from a URL, i think a single subscriber struct is enough

2. id in Program and url in Subscriber is intended to be unique. Explain based on your understanding, is using Vec (list) sufficient or using DashMap (map/dictionary) like we currently use is necessary for this case?

A dashmap allow us to access a subscriber by it's url in a constant fast time because it works like a dictionary, meanwhile searching through a vec might require more time if its a long list. therefore it's more effecient although not necessary for this case to use dictionary of dashmap.

3. When programming using Rust, we are enforced by rigorous compiler constraints to make a thread-safe program. In the case of the List of Subscribers (SUBSCRIBERS) static variable, we used the DashMap external library for thread safe HashMap. Explain based on your understanding of design patterns, do we still need DashMap or we can implement Singleton pattern instead?

As far as i know, dashmap is designed to be thread safe, this means itcan handles multiple access with built in thread protection, meanwhile singleton pattern ensure ONLY ONE instance of the subscriber list exist, singleton pattern is not inherently designed to handle multiple request at the same time, which if done incorrectly would corrupt the data. Therefore with my understandng using dashmap to handle multiple access at the same time for something like the static subscriber is simpler and a safer way to mitigate data corruption due to multiple thread at the same time.

#### Reflection Publisher-2

1. In the Model-View Controller (MVC) compound pattern, there is no “Service” and “Repository”. Model in MVC covers both data storage and business logic. Explain based on your understanding of design principles, why we need to separate “Service” and “Repository” from a Model?

Imagine a kitchen, in this kitchen the model is just a recipe sheet (what ingredient we need to make something, in this case like the url and name), the repository is like the kitchen staff who knows exactly where to get the ingredients and where to store finished food (saving and loading data respectively), and service is the chefs who reads the recipe, tell the kitchen staff what to get, follow the cookings step to produce the food. With this separation of concern, we make sure no one part of the kitchen get overworked/have to many responsibility, not only does this make it more easy to read/modify/maintain, its also easier to swap out parts (for example, changing database).

2. What happens if we only use the Model? Explain your imagination on how the interactions between each model (Program, Subscriber, Notification) affect the code complexity for each model?

Imagine if the recipe sheet (model) also had instructions for where every ingredient is in the kitchen, and detailed steps for how to process ingredients, and how to arrange it on a plate, and notes about what other recipes use the same ingredients, it would mean each model (Product, Subscriber) would be super long and complicated because trying to do everything by themselves. If we want to change how we store Subscribers, we might have to go edit the Product model too because it somehow knows too much about subscribers. Everything gets tangled, making it a headache to change anything without breaking something else, which is something separation of concern principle is designed to prevent in the first place.


3. Have you explored more about Postman? Tell us how this tool helps you to test your current work. You might want to also list which features in Postman you are interested in or feel like it is helpful to help your Group Project or any of your future software engineering projects.

Postman is basically a tool to poke at our web server and see if it does what its programmed to do without needing a fancy webpage we can interact with, for example, we can tell it "hey server, try subscribing with this info" (i.e. sending a POST request with data), and Postman will show us what the server sends back. Postman tool is an awesome way to check if our backend code work without having to implement every part (i.e. frontend part).

What postman do:
- keep all tests organized
- test multiple part at once
- makes group projects way easier because everyone can use the same tests.

#### Reflection Publisher-3

1. Observer Pattern has two variations: Push model (publisher pushes data to subscribers) and Pull model (subscribers pull data from publisher). In this tutorial case, which variation of Observer Pattern that we use?

From what i know, in this tutorial we're using the Push model, i.e. when something happen, the NotificationService will gather all information the subscriber might need into a notification, and directly push it to the subscriber, meanwhile the subscriber can just wait and receive the data.

2. What are the advantages and disadvantages of using the other variation of Observer Pattern for this tutorial case? (example: if you answer Q1 with Push, then imagine if we used Pull)

Unlike the answer to the previous question, if we use pull instead, then the NotificationService would call the subscriber informing it that something has changed, then it would be the subscriber job to go and ask/pull the information of what changed.

the pro of pull is the subscriber could choose exactly what infor they want to pull, if they don't need everything, this can reduce the info sent

the con of pull is that it's more work overall, the subscriber need extra code to ask detail to the publisher after they've been informed that a change has occur, this iform-ask-detail back and forth is adding to the network traffic. 

3. Explain what will happen to the program if we decide to not use multi-threading in the notification process.

If we didn't use threading, the notify function would have to send notification one by one, for example it would have to call update on the first subscriber, wait for the http request to finish, and then it can call update on the subscriber, and wait for that http request to finisih before moving on to the third subscriber, and so on. As one could've imaged, if we have a lot of subscriber, this process would take a very long time, instead we can use threading to let all tose notification http requests happen at the same time, making it wayyyy faster.